function [crlb]=CRLB(txKn,tx,sxKn,sx,Q_TDOA,Q_txAOA,Q_sxAOA)
% [crlb]=CRLB(txKn,tx,sxKn,sx,Q_TDOA,Q_txAOA,Q_sxAOA)
%
% This function obtains the CRLB for the transmitters and scatterers
% in the presence of intentional transmitters and reflectors at known
% positions.
%
% Input
% txKn:     ((Mbar*Dim) x 1), known positions of intentional transmitters.
% tx:       ((M*Dim) x 1), unknown transmitter positions.
% sxKn:     ((Nbar*Dim) x 1), known positions of intentional scatterers.
% sx:       ((N*Dim) x 1), unknown scatterer positions.
% Q_TDOA:   TDOA covariance matrix.
% Q_txAOA:  AOA covariance matrix of unknown transmitters.
% Q_txAOA:  AOA covariance matrix of unknown scatterers.
%
%
% Output
% crlb:     CRLB matrix of unknown transmitter and scatter positions
%
% The routine can be used for 2D(Dim=2) or 3D(Dim=3) localization.
%
% Reference:
% Reference: Y. Zhang and K. C. Ho, "Localization of transmitters and scatterers by single 
% receiver", IEEE Trans. Signal Process., vol. 71, pp. 2267-2282, 2023.
%
% Yang Zhang, K. C. Ho     06-28-2023
%
%       Copyright (C) 2021
%       Computational Intelligence Signal Processing Laboratory
%       University of Missouri
%       Columbia, MO 65211, USA.
%       hod@missouri.edu
%

Mbar=size(txKn,2);  % number of intentional transmitters at known positions
Nbar=size(sxKn,2);  % number of intentional scatterers at known positions
M=size(tx,2)-Mbar;  % number of transmitters at unknown positions
N=size(sx,2)-Nbar;  % number of scatterers at unknown positions

if (size(Q_txAOA,1)>M) K=3; else K=2; end;  % localization dimension

if (Mbar~=0)
    for i=1:Mbar
        for j=1:N
            rho_ds_omega(K*M+K*(j-1)+1:K*M+K*j,j+(i-1)*N)=(sx(:,Nbar+j)-txKn(:,i))/norm(txKn(:,i)-sx(:,Nbar+j))+sx(:,Nbar+j)/norm(sx(:,Nbar+j));
        end
    end
else
    rho_ds_omega=[];
end;

if (Nbar~=0)
    for i=1:M
        for j=1:Nbar
            rho_dt_tx(K*(i-1)+1:K*i,j+(i-1)*Nbar)=(tx(:,Mbar+i)-sxKn(:,j))/norm(tx(:,Mbar+i)-sxKn(:,j))-tx(:,Mbar+i)/norm(tx(:,Mbar+i));
        end
    end
    rho_dt_omega=[rho_dt_tx;zeros(K*N,M*Nbar)];
else
    rho_dt_omega=[];
end;

for i=1:M
    for j=1:N
        rho_dts_omega(K*(i-1)+1:K*i,j+(i-1)*N)=(tx(:,Mbar+i)-sx(:,Nbar+j))/norm(tx(:,Mbar+i)-sx(:,Nbar+j))-tx(:,Mbar+i)/norm(tx(:,Mbar+i));
        rho_dts_omega(K*M+K*(j-1)+1:K*M+K*j,j+(i-1)*N)=(sx(:,Nbar+j)-tx(:,Mbar+i))/norm(tx(:,Mbar+i)-sx(:,Nbar+j))+sx(:,Nbar+j)/norm(sx(:,Nbar+j));
    end
end

rho_d_omega=[rho_ds_omega,rho_dt_omega,rho_dts_omega];


for i=1:M
    rho_theta1_tx(K*(i-1)+1:K*i,i)=[-tx(2,Mbar+i),tx(1,Mbar+i),zeros(1,K-2)]./((norm([tx(2,Mbar+i),tx(1,Mbar+i)]))^2);
    if K==3
        rho_theta2_tx(K*(i-1)+1:K*i,i)=[(-tx(3,Mbar+i)/(norm(tx(:,Mbar+i)))^2)*(tx(1,Mbar+i)/(norm([tx(2,Mbar+i),tx(1,Mbar+i)]))),(-tx(3,Mbar+i)/(norm(tx(:,Mbar+i)))^2)*(tx(2,Mbar+i)/(norm([tx(2,Mbar+i),tx(1,Mbar+i)]))),(norm([tx(2,Mbar+i),tx(1,Mbar+i)]))/(norm(tx(:,Mbar+i)))^2];
    end
end

if K==2
    rho_angleTx_omega=[rho_theta1_tx;zeros(K*N,M)];
else
    rho_angleTx_omega=[rho_theta1_tx,rho_theta2_tx;zeros(K*N,2*M)];
end

for j=1:N
    rho_theta1_sx(K*(j-1)+1:K*j,j)=[-sx(2,Nbar+j),sx(1,Nbar+j),zeros(1,K-2)]./((norm([sx(2,Nbar+j),sx(1,Nbar+j)]))^2);
    if K==3
        rho_theta2_sx(K*(j-1)+1:K*j,j)=[(-sx(3,Nbar+j)/(norm(sx(:,Nbar+j)))^2)*(sx(1,Nbar+j)/(norm([sx(2,Nbar+j),sx(1,Nbar+j)]))),(-sx(3,Nbar+j)/(norm(sx(:,Nbar+j)))^2)*(sx(2,Nbar+j)/(norm([sx(2,Nbar+j),sx(1,Nbar+j)]))),(norm([sx(2,Nbar+j),sx(1,Nbar+j)]))/(norm(sx(:,Nbar+j)))^2];
    end
end

if K==2
    rho_angleSx_omega=[zeros(K*M,N);rho_theta1_sx];
else
    rho_angleSx_omega=[zeros(K*M,2*N);rho_theta1_sx,rho_theta2_sx];
end

FIM=rho_d_omega*inv(Q_TDOA)*rho_d_omega'+rho_angleTx_omega*inv(Q_txAOA)*rho_angleTx_omega'+rho_angleSx_omega*inv(Q_sxAOA)*rho_angleSx_omega';
crlb=inv(FIM);

%end




