function [omega,costML]=GN_MLE(txKn,sxKn,ds,dt,dts,angleTx,angleSx,Q,omega0)
% omega=GN_MLE(txKn,sxKn,ds,dt,dts,angleTx,angleSx,Q,omega0)
%
% This function implements the MLE using the Gauss-Newton iteration to
% obtain the object location estimate.
%
% Input
% txKn:     ((Mbar*dim) x 1), known transmitter position vector
% sxKn:     ((Nbar*dim) x 1), known scatterer position vector
% ds:       TDOAs from known transmitter and unknown scatterer
% dt:       TDOAs from unknown transmitter and known scatterer
% dts:       TDOAs from unknown transmitter and unknown scatterer
% angleTx:  transmitter angles
% angleSx:  scatterer angles
% Q:        measurement covariance matrix
% omega0:   initial solution guess
%
% Output
% omega:    estimate of the unknown transmitter and scatterer positions
%
% The routine can be used for 2D(Dim=2) or 3D(Dim=3) localization.
%
% Reference:
% Reference: Y. Zhang and K. C. Ho, "Localization of transmitters and scatterers by single 
% receiver", IEEE Trans. Signal Process., vol. 71, pp. 2267-2282, 2023.
%
% Yang Zhang, K. C. Ho     06-28-2023
%
%       Copyright (C) 2022
%       Computational Intelligence Signal Processing Laboratory
%       University of Missouri
%       Columbia, MO 65211, USA.
%       hod@missouri.edu
%

K=size(angleTx,1)+1;   % localization dimension
Mbar=size(txKn,2);
Nbar=size(sxKn,2);
M=size(angleTx,2); 
N=size(angleSx,2);

iQ=inv(Q);

omega=omega0;
imax=10;

for ii=1:1:imax
    tx=reshape(omega(1:K*M),K,M);
    sx=reshape(omega(K*M+1:end),K,N);
    
    for j=1:N
        if (Mbar~=0)
            for i=1:Mbar
                d1(i,j)=norm(txKn(:,i)-sx(:,j))+norm(sx(:,j))-norm(txKn(:,i));
                rho_d1_omega(K*M+K*(j-1)+1:K*M+K*j,j+(i-1)*N)=(sx(:,j)-txKn(:,i))/norm(txKn(:,i)-sx(:,j))+sx(:,j)/norm(sx(:,j));
            end;
        else 
            rho_d1_omega=[]; d1=[];
        end;
        angleSxHat(1,j)=atan2(sx(2,j),sx(1,j));
        if K==3
            angleSxHat(2,j)=asin(sx(3,j)/norm(sx(:,j)));
        end
    end
    
    for i=1:M
        if (Nbar~=0)
            for j=1:Nbar
                d2(i,j)=norm(tx(:,i)-sxKn(:,j))+norm(sxKn(:,j))-norm(tx(:,i));
                rho_d2_tx(K*(i-1)+1:K*i,j+(i-1)*Nbar)=(tx(:,i)-sxKn(:,j))/norm(tx(:,i)-sxKn(:,j))-tx(:,i)/norm(tx(:,i));
            end
        else
            rho_d2_tx=[]; d2=[];
            rho_d2_omega=[];
        end;
        angleTxHat(1,i)=atan2(tx(2,i),tx(1,i));
        if K==3
            angleTxHat(2,i)=asin(tx(3,i)/norm(tx(:,i)));
        end
    end
    if (Nbar~=0)
        rho_d2_omega=[rho_d2_tx;zeros(K*N,M*Nbar)];
    end;

    for i=1:M
        for j=1:N
            d3(i,j)=norm(tx(:,i)-sx(:,j))+norm(sx(:,j))-norm(tx(:,i));
            rho_d3_omega(K*(i-1)+1:K*i,j+(i-1)*N)=(tx(:,i)-sx(:,j))/norm(tx(:,i)-sx(:,j))-tx(:,i)/norm(tx(:,i));
            rho_d3_omega(K*M+K*(j-1)+1:K*M+K*j,j+(i-1)*N)=(sx(:,j)-tx(:,i))/norm(tx(:,i)-sx(:,j))+sx(:,j)/norm(sx(:,j));
        end
    end
    
    rho_d_theta=[rho_d1_omega,rho_d2_omega,rho_d3_omega];
    
    for i=1:M
        rho_theta1Tx_tx(K*(i-1)+1:K*i,i)=[-tx(2,i),tx(1,i),zeros(1,K-2)]./((norm([tx(2,i),tx(1,i)]))^2);
        if K==3
            rho_theta2Tx_tx(K*(i-1)+1:K*i,i)=[(-tx(3,i)/(norm(tx(:,i)))^2)*(tx(1,i)/(norm([tx(2,i),tx(1,i)]))),(-tx(3,i)/(norm(tx(:,i)))^2)*(tx(2,i)/(norm([tx(2,i),tx(1,i)]))),(norm([tx(2,i),tx(1,i)]))/(norm(tx(:,i)))^2];
        end
    end
    
    if K==2
        rho_angleTx_theta=[rho_theta1Tx_tx;zeros(K*N,M)];
    else
        rho_angleTx_theta=[rho_theta1Tx_tx,rho_theta2Tx_tx;zeros(K*N,2*M)];
    end
    
    for j=1:N
        rho_theta1Sx_sx(K*(j-1)+1:K*j,j)=[-sx(2,j),sx(1,j),zeros(1,K-2)]./((norm([sx(2,j),sx(1,j)]))^2);
        if K==3
            rho_theta2Sx_sx(K*(j-1)+1:K*j,j)=[(-sx(3,j)/(norm(sx(:,j)))^2)*(sx(1,j)/(norm([sx(2,j),sx(1,j)]))),(-sx(3,j)/(norm(sx(:,j)))^2)*(sx(2,j)/(norm([sx(2,j),sx(1,j)]))),(norm([sx(2,j),sx(1,j)]))/(norm(sx(:,j)))^2];
        end
    end
    
    if K==2
        rho_angleSx_theta=[zeros(K*M,N);rho_theta1Sx_sx];
    else
        rho_angleSx_theta=[zeros(K*M,2*N);rho_theta1Sx_sx,rho_theta2Sx_sx];
    end
    
    J=[rho_d_theta,rho_angleTx_theta,rho_angleSx_theta];
    
    for i=1:M
        if abs(angleTx(1,i)-angleTxHat(1,i))>=pi/2 && sign(angleTx(1,i))~=sign(angleTxHat(1,i))
            if abs(angleTx(1,i)-pi)<=10/180*pi
                angleTxHat(1,i)=angleTxHat(1,i)+2*pi;
            elseif abs(angleTx(1,i)+pi)<=10/180*pi
                angleTxHat(1,i)=angleTxHat(1,i)-2*pi;
            end
        end
    end
    
    for i=1:N
        if abs(angleSx(1,i)-angleSxHat(1,i))>=pi/2 && sign(angleSx(1,i))~=sign(angleSxHat(1,i))
            if abs(angleSx(1,i)-pi)<=10/180*pi
                angleSxHat(1,i)=angleSxHat(1,i)+2*pi;
            elseif abs(angleSx(1,i)+pi)<=10/180*pi
                angleSxHat(1,i)=angleSxHat(1,i)-2*pi;
            end
        end
    end
    
    err=[reshape(ds',[],1)-reshape(d1',[],1);reshape(dt',[],1)-reshape(d2',[],1);reshape(dts',[],1)-reshape(d3',[],1);reshape(angleTx',[],1)-reshape(angleTxHat',[],1);reshape(angleSx',[],1)-reshape(angleSxHat',[],1)];
    ttxp=omega;
    omega=omega+inv(J*iQ*J')*J*iQ*err;
    if norm(omega-ttxp)<(1e-4)
        break;
    end
end

costML=err'*iQ*err;     % ML cost function
