function [Qlty_1,Qlty_Sx]=IndSolSelKnTx(Tx1,Qsub,alphaSol,M,N)
% [Qlty_1,Qlty_Sx]=IndSolSelKnTx(Tx1,Qsub,alphaSol,M,N)
%
% This routine selects the individual minimal measurement solutions
% by intentional transmitters for BLUE to obtain the final solution.
%
% Input   
% Tx1:      positions of intentional transmitters
% Qsub:     measurement noise covariance matrix
% alphaSol: unknown transmitter and scatterer solutions by intentional transmitters
% M:        number of unknown transmitters
% N:    total number of transmitters
%
% Output   
% Qlty_1:   quality factor for solutions by intentional transmitters
% Qlty_Sx:  quality factor for scatterer solutions by intentional transmitters
%
% The routine can be used for 2D(K=2) or 3D(K=3) localization.
%
% Reference:
% Reference: Y. Zhang and K. C. Ho, "Localization of transmitters and scatterers by single 
% receiver", IEEE Trans. Signal Process., vol. 71, pp. 2267-2282, 2023.
%
% Yang Zhang, K. C. Ho     06-28-2023
%
%       Copyright (C) 2023
%       Computational Intelligence Signal Processing Laboratory
%       University of Missouri
%       Columbia, MO 65211, USA.
%       hod@missouri.edu
%
       
%K=size(Tx1,1);
[K,Mbar]=size(Tx1);
Tx_est=reshape(alphaSol(1:K*M),K,M);
Sx_est=reshape(alphaSol(K*M+1:end),K,N);

for i=1:Mbar
    for j=1:N
        for k=1:M
            Txi=Tx1(:,i);
            SolTx=Tx_est(:,k);
            SolSx=Sx_est(:,j);
            alpha=atan2(SolTx(2),SolTx(1));
            beta=atan2(SolSx(2),SolSx(1));
            if K==3
                thetaSx=asin(SolSx(3)/norm(SolSx));
                thetaTx=asin(SolTx(3)/norm(SolTx));
            end
            rho_d=[zeros(1,K),((SolSx-Txi)/norm(Txi-SolSx)+SolSx/norm(SolSx))';
                ((SolTx-SolSx)/norm(SolTx-SolSx)-SolTx/norm(SolTx))',((SolSx-SolTx)/norm(SolTx-SolSx)+SolSx/norm(SolSx))'];
            
            rho_angleTx=[[-SolTx(2),SolTx(1),zeros(1,K-2)]./((norm([SolTx(2),SolTx(1)]))^2),zeros(1,K)];
            rho_angleSx=[zeros(1,K),[-SolSx(2),SolSx(1),zeros(1,K-2)]./((norm([SolSx(2),SolSx(1)]))^2)];
            if K==3
                rho_angleTx=[rho_angleTx;[-sin(thetaTx)*cos(alpha),-sin(thetaTx)*sin(alpha),cos(thetaTx)]/norm(SolTx),zeros(1,K)];
                rho_angleSx=[rho_angleSx;zeros(1,K),[-sin(thetaSx)*cos(beta),-sin(thetaSx)*sin(beta),cos(thetaSx)]/norm(SolSx)];
            end
            
            B_rhoi=[rho_d;rho_angleTx;rho_angleSx];
            CovInd=inv(B_rhoi)*Qsub*inv(B_rhoi)';
            % FIMInd=B_rhoi'*inv(Qsub)*B_rhoi;
            Qlty_1((i-1)*N+j,k)=trace(CovInd);
            Qlty_Sx(i,j)=trace(CovInd(K+1:2*K,K+1:2*K));
        end
    end
end









