function Indtr=IndvSolSel(Qlty_1,Qlty_2,Qlty_3,M,N,CsolTx,CsolSx)
% Indtr=IndvSolSel(Qlty_1,Qlty_2,Qlty_3,M,N,CsolTx,CsolSx)
%
% This routine selects the individual minimal measurement solutions
% for BLUE to obtain the final solution.
%
% Input   
% Qlty_1:   quality factor for solutions by intentional transmitters
% Qlty_2:   quality factor for solutions by intentional scatterers
% Qlty_3:   quality factor for solutions by unknown transmitters and scatterers
% M:        number of unknown transmitters
% N:        number of unknown scatterers
% Q:        measurement noise covariance matrix
% CsolTx, CsolSx:   combinations of tx and sx in computing unknown transmitter and scatterer solutions
% Indtr:    individual solution selection indicator
%
% Output   
% Indtr:    individual solution selection indicator
%
% The routine can be used for 2D(K=2) or 3D(K=3) localization.
%
% Reference:
% Reference: Y. Zhang and K. C. Ho, "Localization of transmitters and scatterers by single 
% receiver", IEEE Trans. Signal Process., vol. 71, pp. 2267-2282, 2023.
%
% Yang Zhang, K. C. Ho     06-28-2023
%
%       Copyright (C) 2023
%       Computational Intelligence Signal Processing Laboratory
%       University of Missouri
%       Columbia, MO 65211, USA.
%       hod@missouri.edu
%
       
Qlty_1=Qlty_1/4;
Qlty_2=Qlty_2/4;
Qlty_3=Qlty_3/8;

BigNum=1e99;
Indtr=[];
d3Indtr=zeros(M,N);

while(prod(prod(d3Indtr))==0)
    
    if (~isempty(Qlty_1))
	[min1r,Ind1]=min(Qlty_1);
        [min1,Ind1c]=min(min1r);
        Ind1r=Ind1(Ind1c);
    else
       min1=BigNum;
    end;

    if (~isempty(Qlty_2))
        [min2r,Ind2]=min(Qlty_2);
        [min2,Ind2c]=min(min2r);
        Ind2r=Ind2(Ind2c);
    else
        min2=BigNum;
    end;

    if isempty(Qlty_3)
        [~,Ind]=min([min1,min2]);
    else
        [min3,Ind3]=min(Qlty_3);
        [~,Ind]=min([min1,min2,min3]);
    end

    if Ind==1      
        ii=ceil(Ind1r/(N));
        jj=Ind1r-(ii-1)*N;
        kk=Ind1c;
        if d3Indtr(kk,jj)==1
            Qlty_1(Ind1r,Ind1c)=Inf;
            continue;
        end
        Indtr=[Indtr,[1;ii;jj;kk]];
        Qlty_1(Ind1r,Ind1c)=Inf;
        d3Indtr(kk,jj)=1;
        
    elseif Ind==2
        ii=ceil(Ind2r/(M));
        jj=Ind2r-(ii-1)*M;
        kk=Ind2c;
        if d3Indtr(jj,kk)==1
            Qlty_2(Ind2r,Ind2c)=Inf;
            continue;
        end
            
        Indtr=[Indtr,[2;ii;jj;kk]];
        Qlty_2(Ind2r,Ind2c)=Inf;
        d3Indtr(jj,kk)=1;
  
    else
        ii=ceil((Ind3)/nchoosek(N,2));
        jj=(Ind3)-(ii-1)*nchoosek(N,2);
        if d3Indtr(CsolTx(ii,1),CsolSx(jj,1))==1 && d3Indtr(CsolTx(ii,1),CsolSx(jj,2))==1 ...
                && d3Indtr(CsolTx(ii,2),CsolSx(jj,1))==1 && d3Indtr(CsolTx(ii,2),CsolSx(jj,2))==1
            Qlty_3(Ind3)=Inf;
            continue;
        end
        Indtr=[Indtr,[3;ii;jj;0]];
        Qlty_3(Ind3)=Inf;
        d3Indtr(CsolTx(ii,1),CsolSx(jj,1))=1;
        d3Indtr(CsolTx(ii,1),CsolSx(jj,2))=1;
        d3Indtr(CsolTx(ii,2),CsolSx(jj,1))=1;
        d3Indtr(CsolTx(ii,2),CsolSx(jj,2))=1;
    end
end

%end
    
    
    

