function [omegaDef,omegaSel]=MMBLUE(txKn,sxKn,Mbar,M,Nbar,N,ds,dt,dts,angleTx,angleSx,Q,QsubTx,QsubSx,QsubTxSx)
% [omegaDef,omegaSel]=MMBLUE(txKn,sxKn,Mbar,M,Nbar,N,ds,dt,dts,angleTx,angleSx,Q,QsubTx,QsubSx,QsubTxSx)
% 
% This function uses BLUE to combine the minimal measurement solutions to
% obtain the final object location estimate.
%
% Input
% txKn:     ((Mbar*dim) x 1), intentional transmitter position vector
% sxKn:     ((Nbar*dim) x 1), intentional scatterer position vector
% Mbar:     number of intentional transmitters at known positions
% M:        number of unknown transmitters
% Nbar:     number of intentional scatterers at known positions
% N:        number of unknown scatterers
% ds:       TDOAs from intentional transmitters and unknown scatterers
% dt:       TDOAs from unknown transmitters and intentional scatterers
% dts:      TDOAs from unknown transmitters and unknown scatterers
% angleTx:  transmitter AOAs
% angleSx:  scatterer AOAs
% Q:        measurement noise covariance matrix
% QsubTx:  noise covariance matrix for solution by intentional transmitters
% QsubSx:  noise covariance for solution by intentional scatterers
% QsubTxSx: noise covariance for solutions by unknown transmitters and scatterers
%
% Output
% omegaDef: estimate without individual solution selection
% omegaSel: estimate with individual solution selection
%
% The routine can be used for 2D(K=2) or 3D(K=3) localization.
%
% Reference:
% Reference: Y. Zhang and K. C. Ho, "Localization of transmitters and scatterers by single 
% receiver", IEEE Trans. Signal Process., vol. 71, pp. 2267-2282, 2023.
%
% Yang Zhang, K. C. Ho     06-28-2023
%
%       Copyright (C) 2023
%       Computational Intelligence Signal Processing Laboratory
%       University of Missouri
%       Columbia, MO 65211, USA.
%       hod@missouri.edu
%

K=size(angleTx,1)+1;   % localization dimension

theta1Tx=angleTx(1,:);   % transmitter azimuth angle
theta1Sx=angleSx(1,:);    % scatterer azimuth angle

if K==3
    theta2Tx=angleTx(2,:);    % transmitter elevation angle
    theta2Sx=angleSx(2,:);    % scatterer elevation angle
end

% ------------ individual solution by intentional Transmitters ----------- %
solTx_1=[]; solSx_1=[];
for i=1:Mbar
    for j=1:N
        for k=1:M
            d13Ind=[ds(i,j);dts(k,j)];
            [solTx_1(K*(i-1)*N+K*(j-1)+1:K*(i-1)*N+K*j,k),solSx_1(K*(i-1)+1:K*i,j),Qlty_1((i-1)*N+j,k),QltyTx_1((i-1)*N+j,k),QltySx_1(i,j)]=MinSolKnTx(txKn(:,i),d13Ind,angleTx(:,k),angleSx(:,j),QsubTx);
            % solTx_1, solSx_1 : transmitter and scatterer position estimates
            % Qlty_1 : individual solution quality
            % QltyTx_1 : individual transmitter solution quality
            % QltySx_1 : individual scatterer solution quality
        end
    end
end

% ------------ individual solution by intentional Scatterers  ----------- %
solTx_2=[]; solSx_2=[];
for i=1:Nbar
    for j=1:M
        for k=1:N
            dt3Ind=[dt(j,i);dts(j,k)];
            [solTx_2(K*(i-1)+1:K*i,j),solSx_2(K*(i-1)*M+K*(j-1)+1:K*(i-1)*M+K*j,k),Qlty_2((i-1)*M+j,k),QltyTx_2(i,j),QltySx_2((i-1)*M+j,k)]=MinSolKnSx(sxKn(:,i),dt3Ind,angleTx(:,j),angleSx(:,k),QsubSx);
            % solTx_2, solSx_2 : transmitter and scatterer position estimates
            % Qlty_2 : individual solution quality
            % QltyTx_2 : individual transmitter solution quality
            % QltySx_2 : individual scatterer solution quality
        end
    end
end

% ------------ individual solution by unknown transmitters and scatterer ------------ %

Qlty_3=[];
CsolTx=[];
CsolSx=[];
sol=[];

if M>=2 && N>=2    % this individual solution exists only when M >= 2 & N >= 2

    SoRtx=1:M;
    CsolTx=nchoosek(SoRtx,2);                 % possible combinations of tx
    SoRsx=1:N;
    CsolSx=nchoosek(SoRsx,2);                 % possible combinations of sx
    [NumTx,~]=size(CsolTx);                   % number of tx combination
    [NumSx,~]=size(CsolSx);                   % number of sx combination

    for i=1:NumTx
        for j=1:NumSx
            dInd=dts(CsolTx(i,:),CsolSx(j,:));
            angleTxInd=angleTx(:,CsolTx(i,:));
            angleSxInd=angleSx(:,CsolSx(j,:));
            [sol(:,(i-1)*NumSx+j),Qlty_3((i-1)*NumSx+j)]=MinSolUnTxUnSx(dInd,angleTxInd,angleSxInd,QsubTxSx);
            % sol : transmitter and scatterer position estimates
            % Qlty_3 : solution
        end
    end
end

if (Mbar~=0 | Nbar~=0)

    % ---------------------- default choice of individual solutions ---------------------- %
    for i=1:M
        for j=1:N
            if (Mbar~=0)
                Indtrdefault(:,(i-1)*N+j)=[1;1;i;j];
            elseif (Nbar~=0)
                Indtrdefault(:,(i-1)*N+j)=[2;1;i;j];
            end;
        end
    end

    % --------------------------------- recompute quality -------------------------------- %

    % ----------- obtain best scatterer estimate ------------ %
    for j=1:N

        if (Mbar~=0 & Nbar ~=0)
            [min1,Inds]=min(QltySx_1(:,j));
            [min2,Indt]=min(QltySx_2(:,j));

            [~,Ind]=min([min1,min2]);

            if Ind==1
                sx_est(:,j)=solSx_1(K*(Inds-1)+1:K*Inds,j);
            else
                ii=ceil(Indt/(M));
                jj=Indt-(ii-1)*M;
                sx_est(:,j)=solSx_2(K*(ii-1)*M+K*(jj-1)+1:K*(ii-1)*M+K*jj,j);
            end;
        elseif (Mbar~=0 & Nbar==0)
            [min1,Ind1]=min(QltySx_1(:,j));
            sx_est(:,j)=solSx_1(K*(Ind1-1)+1:K*Ind1,j);
        elseif (Mbar==0 & Nbar~=0)
            [~,Ind2]=min(QltySx_2(:,j));
            ii=ceil(Ind2/(M));
            jj=Ind2-(ii-1)*M;
            sx_est(:,j)=solSx_2(K*(ii-1)*M+K*(jj-1)+1:K*(ii-1)*M+K*jj,j);
        end
    end

    % ----------- obtain best transmitter estimate ------------ %
    for i=1:M

        if (Mbar~=0 & Nbar ~=0)
            [min1,Inds]=min(QltyTx_1(:,i));
            [min2,Indt]=min(QltyTx_2(:,i));

            [~,Ind]=min([min1,min2]);

            if Ind==1
                ii=ceil(Inds/(N));
                jj=Inds-(ii-1)*N;
                tx_est(:,i)=solTx_1(K*(ii-1)*N+K*(jj-1)+1:K*(ii-1)*N+K*jj,i);
            else
                tx_est(:,i)=solTx_2(K*(Indt-1)+1:K*Indt,i);
            end;
        elseif (Mbar~=0 & Nbar==0)
            [min1,Ind1]=min(QltyTx_1(:,i));
            ii=ceil(Ind1/(N));
            jj=Ind1-(ii-1)*N;
            tx_est(:,i)=solTx_1(K*(ii-1)*N+K*(jj-1)+1:K*(ii-1)*N+K*jj,i);
        elseif (Mbar==0 & Nbar~=0)
            [~,Ind2]=min(QltyTx_2(:,i));
            tx_est(:,i)=solTx_2(K*(Ind2-1)+1:K*Ind2,i);
        end
    end

else    % (Mbar==0 & Nbar==0)

    % ---------------------- default choice of individual solutions ---------------------- %
    d3Indtr=zeros(M,N);
    Indtrdefault=[];

    for i=1:NumTx*NumSx

        ii=ceil((i)/nchoosek(N,2));
        jj=i-(ii-1)*nchoosek(N,2);
        if prod(prod(d3Indtr(CsolTx(ii,1:2),CsolSx(jj,1:2))))~=1
            Indtrdefault=[Indtrdefault,[3;ii;jj;0]];
            d3Indtr(CsolTx(ii,1),CsolSx(jj,1))=1;
            d3Indtr(CsolTx(ii,1),CsolSx(jj,2))=1;
            d3Indtr(CsolTx(ii,2),CsolSx(jj,1))=1;
            d3Indtr(CsolTx(ii,2),CsolSx(jj,2))=1;
        end
        if (prod(prod(d3Indtr))~=0)
            break;
        end
    end

    % --------------------------------- recompute quality -------------------------------- %

    % ----------- obtain best transmitter and scatterer estimates ------------ %
    CsolTxSel=[];
    CsolSxSel=[];
    A_tx=[];
    A_sx=[];

    while(length(A_tx)~=M || length(A_sx)~=N)
        [~,idx]=min(Qlty_3);
        i=ceil(idx/(nchoosek(N,2)));
        j=idx-(i-1)*nchoosek(N,2);
        if ~ismember(CsolTx(i,1),A_tx)
            tx_est(:,CsolTx(i,1))=sol(1:K,(i-1)*NumSx+j);
        end
        if ~ismember(CsolTx(i,2),A_tx)
            tx_est(:,CsolTx(i,2))=sol(K+1:2*K,(i-1)*NumSx+j);
        end
        if ~ismember(CsolSx(j,1),A_sx)
            sx_est(:,CsolSx(j,1))=sol(2*K+1:3*K,(i-1)*NumSx+j);
        end
        if ~ismember(CsolSx(j,2),A_sx)
            sx_est(:,CsolSx(j,2))=sol(3*K+1:4*K,(i-1)*NumSx+j);
        end
        A_tx=union(A_tx,CsolTx(i,:));
        A_sx=union(A_sx,CsolSx(j,:));
        Qlty_3(idx)=Inf;
    end

end;


% ---------- solution ----------
omegaEst=[reshape(tx_est,[],1);reshape(sx_est,[],1)];

if (Mbar~=0)
    [Qlty_1,Qlty_sx]=IndSolSelKnTx(txKn,QsubTx,omegaEst,M,N); % recompute individual solution 1 quality factor
else
    Qlty_1=[];
end;
if (Nbar~=0)
    [Qlty_2,Qlty_tx]=IndSolSelKnSx(sxKn,QsubSx,omegaEst,M,N); % recompute individual solution 2 quality factor
else
    Qlty_2=[];
end;

% --------------------------------------------------------------------------------------------- %

if M>=2 && N>=2
    Qlty_3=IndSolSelUnTxUnSx(CsolTx,CsolSx,QsubTxSx,K,omegaEst,M,N,NumTx,NumSx); % recompute individual solution 3 quality factor
end

Indtr=IndvSolSel(Qlty_1,Qlty_2,Qlty_3,M,N,CsolTx,CsolSx); % individual solution selection

omegaDef=MMBLUE_EstSel(txKn,sxKn,Mbar,M,Nbar,N,Q,omegaEst,solTx_1,solSx_1,solTx_2,solSx_2,sol,CsolTx,CsolSx,Indtrdefault);
omegaSel=MMBLUE_EstSel(txKn,sxKn,Mbar,M,Nbar,N,Q,omegaEst,solTx_1,solSx_1,solTx_2,solSx_2,sol,CsolTx,CsolSx,Indtr);


