function omega=MMBLUE_EstSel(txKn,sxKn,Mbar,M,Nbar,N,Q,omegaInit,solTx_1,solSx_1,solTx_2,solSx_2,sol,CsolTx,CsolSx,Indtr)
% omega=MMBLUE_EstSel(txKn,sxKn,Mbar,M,Nbar,N,Q,omegaInit,solTx_1,solSx_1,solTx_2,solSx_2,sol,CsolTx,CsolSx,Indtr)
%
% This routine obtains the estimate of the unknown transmitters and
% scatterers by combining the minimal measurement solutions by BLUE.
%
% Input   
% txKn:     ((Mbar*dim) x 1), intentional transmitter position vector
% sxKn:     ((Nbar*dim) x 1), intentional scatterer position vector
% Mbar:     number of intentional transmitters at known positions
% M:        number of unknown transmitters
% Nbar:     number of intentional scatterers at known positions
% N:        number of unknown scatterers
% Q:        measurement noise covariance matrix
% omegaInit:  initial estimate for computing the weighting matrix
% solTx_1, solSx_1: transmitter and scatterer position estimates by intentional transmitters
% solTx_2, solSx_2: transmitter and scatterer position estimates by intentional scatterers
% sol:      transmitter and scatterer position estimates by unknown transmitters and scatterers
% CsolTx, CsolSx:   combinations of tx and sx in computing unknown transmitter and scatterer solutions
% Indtr:    individual solution selection indicator
%
% Output   
% omega:    estimate of the unknown transmitter and scatterer positions
%
% The routine can be used for 2D(K=2) or 3D(K=3) localization.
%
% Reference:
% Reference: Y. Zhang and K. C. Ho, "Localization of transmitters and scatterers by single 
% receiver", IEEE Trans. Signal Process., vol. 71, pp. 2267-2282, 2023.
%
% Yang Zhang, K. C. Ho     06-28-2023
%
%       Copyright (C) 2023
%       Computational Intelligence Signal Processing Laboratory
%       University of Missouri
%       Columbia, MO 65211, USA.
%       hod@missouri.edu
%
       
Rpn=2;
% K=size(txKn,1);
K=length(omegaInit)/(M+N);  % localization dimension
NumTx=size(CsolTx,1);
NumSx=size(CsolSx,1);

omega=omegaInit;

for kk=1:Rpn
    
    tx_est=reshape(omega(1:K*M),K,M);
    sx_est=reshape(omega(K*M+1:end),K,N);
    B=[];
    C=[];
    G=[];
    h=[];
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% known transmitter and unknown reflector %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    for i=1:Mbar
        for j=1:N
            seen=0;
            for k=1:M
                if ismember([1,i,j,k],Indtr','rows')
                    seen=1;
                    txi=tx_est(:,k);
                    seni=[txKn(:,i),tx_est(:,k)];
                    txj=sx_est(:,j);
                    
                    theta1Tx=atan2(txi(2),txi(1));
                    theta1Sx=atan2(txj(2),txj(1));
                    if K==3
                        theta2Sx=asin(txj(3)/norm(txj));
                        theta2Tx=asin(txi(3)/norm(txi));
                    end
                    
                    rho_d=[zeros(1,K),((txj-seni(:,1))/norm(seni(:,1)-txj)+txj/norm(txj))';
                        ((seni(:,2)-txj)/norm(seni(:,2)-txj)-seni(:,2)/norm(seni(:,2)))',((txj-seni(:,2))/norm(seni(:,2)-txj)+txj/norm(txj))'];
                    rho_angleTx=[[-txi(2),txi(1),zeros(1,K-2)]./((norm([txi(2),txi(1)]))^2),zeros(1,K)];
                    rho_angleSx=[zeros(1,K),[-txj(2),txj(1),zeros(1,K-2)]./((norm([txj(2),txj(1)]))^2)];
                    if K==3
                        rho_angleTx=[rho_angleTx;[-sin(theta2Tx)*cos(theta1Tx),-sin(theta2Tx)*sin(theta1Tx),cos(theta2Tx)]/norm(txi),zeros(1,K)];
                        rho_angleSx=[rho_angleSx;zeros(1,K),[-sin(theta2Sx)*cos(theta1Sx),-sin(theta2Sx)*sin(theta1Sx),cos(theta2Sx)]/norm(txj)];
                    end
                    B_rhoi=[rho_d;rho_angleTx;rho_angleSx];
                    C1_di=zeros(1,Mbar*N+M*Nbar);
                    C1_di((i-1)*N+j)=1;
                    C3_di=zeros(1,M*N);
                    C3_di((k-1)*N+j)=1;
                    C_thetaEMi=zeros(K-1,(K-1)*M);
                    C_thetaEMi(1,k)=1;
                    if K==3
                        C_thetaEMi(2,M+k)=1;
                    end
                    C_theta2Sxi=zeros(K-1,(K-1)*N);
                    C_theta2Sxi(1,j)=1;
                    if K==3
                         C_theta2Sxi(2,N+j)=1;
                    end
                    C_i=blkdiag(C1_di,C3_di,C_thetaEMi,C_theta2Sxi);
                    G_i=zeros(2*K,K*(M+N));
                    G_i(1:K,K*(k-1)+1:K*k)=eye(K);
                    G_i(K+1:2*K,K*M+K*(j-1)+1:K*M+K*j)=eye(K);
                    h_i=[solTx_1(K*(i-1)*N+K*(j-1)+1:K*(i-1)*N+K*j,k);solSx_1(K*(i-1)+1:K*i,j)];
                    B=blkdiag(B,B_rhoi);
                    C=[C;C_i];
                    G=[G;G_i];
                    h=[h;h_i];
                end
            end
            
            if seen==0 
                seen=1;
                
                txi=tx_est(:,k);
                txj=sx_est(:,j);
                
                theta1Sx=atan2(txj(2),txj(1));
                if K==3
                    theta2Sx=asin(txj(3)/norm(txj));
                end
                
                rho_d=((txj-txi(:,1))/norm(txi(:,1)-txj)+txj/norm(txj))';
                rho_angleSx=[-txj(2),txj(1),zeros(1,K-2)]./((norm([txj(2),txj(1)]))^2);
                if K==3
                    rho_angleSx=[rho_angleSx;[-sin(theta2Sx)*cos(theta1Sx),-sin(theta2Sx)*sin(theta1Sx),cos(theta2Sx)]/norm(txj)];
                end
                B_rhoi=[rho_d;rho_angleSx];
                C1_di=zeros(1,Mbar*N+M*(Nbar+N));
                C1_di((i-1)*N+j)=1;
                C_theta2Sxi=zeros(K-1,(K-1)*(M+N));
                C_theta2Sxi(1,(K-1)*M+j)=1;
                if K==3
                    C_theta2Sxi(2,(K-1)*M+N+j)=1;
                end
                C_i=blkdiag(C1_di,C_theta2Sxi);
                G_i=zeros(K,K*(M+N));
                G_i(:,K*M+K*(j-1)+1:K*M+K*j)=eye(K);
                h_i=solSx_1(K*(i-1)+1:K*i,j);
                B=blkdiag(B,B_rhoi);
                C=[C;C_i];
                G=[G;G_i];
                h=[h;h_i];
            end
        end
    end
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% known reflector and unknown transmitter %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    for i=1:Nbar
        for j=1:M
            seen=0;
            for k=1:N
                if ismember([2,i,j,k],Indtr','rows')
                    seen=1;
                    
                    seni=tx_est(:,j);
                    txj=[sxKn(:,i),sx_est(:,k)];
                    sxj=sx_est(:,k);
                    theta1Tx=atan2(seni(2),seni(1));
                    theta1Sx=atan2(sxj(2),sxj(1));
                    if K==3
                        theta2Sx=asin(sxj(3)/norm(sxj));
                        theta2Tx=asin(seni(3)/norm(seni));
                    end
                    
                    rho_d=[((seni-txj(:,1))/norm(seni-txj(:,1))-seni/norm(seni))',zeros(1,K);
                        ((seni-txj(:,2))/norm(seni-txj(:,2))-seni/norm(seni))',((txj(:,2)-seni)/norm(seni-txj(:,2))+txj(:,2)/norm(txj(:,2)))'];
                    rho_angleTx=[[-seni(2),seni(1),zeros(1,K-2)]./((norm([seni(2),seni(1)]))^2),zeros(1,K)];
                    rho_angleSx=[zeros(1,K),[-sxj(2),sxj(1),zeros(1,K-2)]./((norm([sxj(2),sxj(1)]))^2)];
                    if K==3
                        rho_angleTx=[rho_angleTx;[-sin(theta2Tx)*cos(theta1Tx),-sin(theta2Tx)*sin(theta1Tx),cos(theta2Tx)]/norm(seni),zeros(1,K)];
                        rho_angleSx=[rho_angleSx;zeros(1,K),[-sin(theta2Sx)*cos(theta1Sx),-sin(theta2Sx)*sin(theta1Sx),cos(theta2Sx)]/norm(sxj)];
                    end
                    B_rhoi=[rho_d;rho_angleTx;rho_angleSx];
                    C2_di=zeros(1,Mbar*N+M*Nbar);
                    C2_di(Mbar*N+(j-1)*Nbar+i)=1;
                    C3_di=zeros(1,M*N);
                    C3_di((j-1)*N+k)=1;
                    C_thetaEMi=zeros(K-1,(K-1)*M);
                    C_thetaEMi(1,j)=1;
                    if K==3
                        C_thetaEMi(2,M+j)=1;
                    end
                    C_theta2Sxi=zeros(K-1,(K-1)*N);
                    C_theta2Sxi(1,k)=1;
                    if K==3
                        C_theta2Sxi(2,N+k)=1;
                    end
                    C_i=blkdiag(C2_di,C3_di,C_thetaEMi,C_theta2Sxi);
                    G_i=zeros(2*K,K*(M+N));
                    G_i(1:K,K*(j-1)+1:K*j)=eye(K);
                    G_i(K+1:2*K,K*M+K*(k-1)+1:K*M+K*k)=eye(K);
                    h_i=[solTx_2(K*(i-1)+1:K*i,j);solSx_2(K*(i-1)*M+K*(j-1)+1:K*(i-1)*M+K*j,k)];
                    B=blkdiag(B,B_rhoi);
                    C=[C;C_i];
                    G=[G;G_i];
                    h=[h;h_i];
                end
            end
            if seen==0 
                seen=1;
                
                seni=tx_est(:,j);
                txj=sxKn(:,i);
        
                theta1Tx=atan2(seni(2),seni(1));
                if K==3
                    theta2Tx=asin(seni(3)/norm(seni));
                end
                
                rho_d=((seni-txj)/norm(seni-txj)-seni/norm(seni))';
                rho_angleTx=[-seni(2),seni(1),zeros(1,K-2)]./((norm([seni(2),seni(1)]))^2);
                if K==3
                    rho_angleTx=[rho_angleTx;[-sin(theta2Tx)*cos(theta1Tx),-sin(theta2Tx)*sin(theta1Tx),cos(theta2Tx)]/norm(seni)];
                end
                B_rhoi=[rho_d;rho_angleTx];
                C2_di=zeros(1,Mbar*N+M*(Nbar+N));
                C2_di(Mbar*N+(j-1)*Nbar+i)=1;
                C_thetaEMi=zeros(K-1,(K-1)*(M+N));
                C_thetaEMi(1,j)=1;
                if K==3
                    C_thetaEMi(2,M+j)=1;
                end
                C_i=blkdiag(C2_di,C_thetaEMi);
                G_i=zeros(K,K*(M+N));
                G_i(:,K*(j-1)+1:j*K)=eye(K);
                h_i=solTx_2(K*(i-1)+1:K*i,j);
                B=blkdiag(B,B_rhoi);
                C=[C;C_i];
                G=[G;G_i];
                h=[h;h_i];
            end
        end
    end

    % ------------------------- unknown reflector and unknown transmitter  ------------------------- 
    for i=1:NumTx
        for j=1:NumSx
            if ismember([3,i,j,0],Indtr','rows')
                seni=tx_est(:,CsolTx(i,:));
                txj=sx_est(:,CsolSx(j,:));
                theta1Tx=[atan2(seni(2,1),seni(1,1)),atan2(seni(2,2),seni(1,2))];
                theta1Sx=[atan2(txj(2,1),txj(1,1)),atan2(txj(2,2),txj(1,2))];
                if K==3
                    theta2Sx=[asin(txj(3,1)/norm(txj(:,1))),asin(txj(3,2)/norm(txj(:,2)))];
                    theta2Tx=[asin(seni(3,1)/norm(seni(:,1))),asin(seni(3,2)/norm(seni(:,2)))];
                end
                
                rho_d=[((seni(:,1)-txj(:,1))/norm(seni(:,1)-txj(:,1))-seni(:,1)/norm(seni(:,1)))',zeros(1,K),((txj(:,1)-seni(:,1))/norm(seni(:,1)-txj(:,1))+txj(:,1)/norm(txj(:,1)))',zeros(1,K);
                    ((seni(:,1)-txj(:,2))/norm(seni(:,1)-txj(:,2))-seni(:,1)/norm(seni(:,1)))',zeros(1,K),zeros(1,K),((txj(:,2)-seni(:,1))/norm(seni(:,1)-txj(:,2))+txj(:,2)/norm(txj(:,2)))';
                    zeros(1,K),((seni(:,2)-txj(:,1))/norm(seni(:,2)-txj(:,1))-seni(:,2)/norm(seni(:,2)))',((txj(:,1)-seni(:,2))/norm(seni(:,2)-txj(:,1))+txj(:,1)/norm(txj(:,1)))',zeros(1,K);
                    zeros(1,K),((seni(:,2)-txj(:,2))/norm(seni(:,2)-txj(:,2))-seni(:,2)/norm(seni(:,2)))',zeros(1,K),((txj(:,2)-seni(:,2))/norm(seni(:,2)-txj(:,2))+txj(:,2)/norm(txj(:,2)))'];
                rho_angleTx=[[-seni(2,1),seni(1,1),zeros(1,K-2)]./(norm([seni(2,1),seni(1,1)])^2),zeros(1,K),zeros(1,2*K);
                    zeros(1,K),[-seni(2,2),seni(1,2),zeros(1,K-2)]./(norm([seni(2,2),seni(1,2)])^2),zeros(1,2*K)];
                rho_angleSx=[zeros(1,2*K),[-txj(2,1),txj(1,1),zeros(1,K-2)]./(norm([txj(2,1),txj(1,1)])^2),zeros(1,K);
                    zeros(1,2*K),zeros(1,K),[-txj(2,2),txj(1,2),zeros(1,K-2)]./(norm([txj(2,2),txj(1,2)])^2)];
                if K==3
                    rho_angleTx2=[[-sin(theta2Tx(1))*cos(theta1Tx(1)),-sin(theta2Tx(1))*sin(theta1Tx(1)),cos(theta2Tx(1))]/norm(seni(:,1)),zeros(1,K),zeros(1,2*K);
                        zeros(1,K),[-sin(theta2Tx(2))*cos(theta1Tx(2)),-sin(theta2Tx(2))*sin(theta1Tx(2)),cos(theta2Tx(2))]/norm(seni(:,2)),zeros(1,2*K)];
                    rho_angleSx2=[zeros(1,2*K),[-sin(theta2Sx(1))*cos(theta1Sx(1)),-sin(theta2Sx(1))*sin(theta1Sx(1)),cos(theta2Sx(1))]/norm(txj(:,1)),zeros(1,K);
                        zeros(1,2*K),zeros(1,K),[-sin(theta2Sx(2))*cos(theta1Sx(2)),-sin(theta2Sx(2))*sin(theta1Sx(2)),cos(theta2Sx(2))]/norm(txj(:,2))];
                    
                    rho_angleTx=[rho_angleTx;rho_angleTx2];
                    rho_angleSx=[rho_angleSx;rho_angleSx2];
                end
                B_rhoi=[rho_d;rho_angleTx;rho_angleSx];
                C3_di=zeros(4,Mbar*N+M*Nbar+M*N);
                C3_di(1,Mbar*N+M*Nbar+(CsolTx(i,1)-1)*N+CsolSx(j,1))=1;
                C3_di(2,Mbar*N+M*Nbar+(CsolTx(i,1)-1)*N+CsolSx(j,2))=1;
                C3_di(3,Mbar*N+M*Nbar+(CsolTx(i,2)-1)*N+CsolSx(j,1))=1;
                C3_di(4,Mbar*N+M*Nbar+(CsolTx(i,2)-1)*N+CsolSx(j,2))=1;
                C_thetaEMi=zeros(2*(K-1),(K-1)*M);
                C_thetaEMi(1,CsolTx(i,1))=1;
                C_thetaEMi(2,CsolTx(i,2))=1;
                if K==3
                    C_thetaEMi(3,M+CsolTx(i,1))=1;
                    C_thetaEMi(4,M+CsolTx(i,2))=1;
                end
                C_theta2Sxi=zeros(2*(K-1),(K-1)*N);
                C_theta2Sxi(1,CsolSx(j,1))=1;
                C_theta2Sxi(2,CsolSx(j,2))=1;
                if K==3
                    C_theta2Sxi(3,N+CsolSx(j,1))=1;
                    C_theta2Sxi(4,N+CsolSx(j,2))=1;
                end
                C_i=blkdiag(C3_di,C_thetaEMi,C_theta2Sxi);
                G_i=zeros(4*K,K*(M+N));
                G_i(1:K,(CsolTx(i,1)-1)*K+1:CsolTx(i,1)*K)=eye(K);
                G_i(K+1:2*K,(CsolTx(i,2)-1)*K+1:CsolTx(i,2)*K)=eye(K);
                G_i(2*K+1:3*K,K*M+(CsolSx(j,1)-1)*K+1:K*M+CsolSx(j,1)*K)=eye(K);
                G_i(3*K+1:4*K,K*M+(CsolSx(j,2)-1)*K+1:K*M+CsolSx(j,2)*K)=eye(K);
                h_i=sol(:,(i-1)*NumSx+j);
                B=blkdiag(B,B_rhoi);
                C=[C;C_i];
                G=[G;G_i];
                h=[h;h_i];
            end
        end
    end
    
    W=B'*pinv(C*Q*C')'*B;
    %     W=pinv(BC*Q*BC')';
    omega=inv(G'*W*G)*G'*W*h;
%     omega=(G'*W*G)\(G')*W*h;

end