function [solTx,solSx,Qlty,QltyTx,QltySx]=MinSolKnTx(TxKn,d,angleTx,angleSx,Qsub)
% [solTx,solSx,Qlty,QltyTx,QltySx]=MinSolKnwnTx(TxKn,d,angleTx,angleSx,Qsub)
%
% This routine obtains the minimal measurement solution of a transmitter
% and a scatter by using one intentional transmitter at a known position. 
%
% Input        
% TxKn:     position vector of intentional transmitters
% d:        TDOA ds and dts measurements
% angleTx:  transmitter AOA measurements
% angleSx:  scatterer AOA measurements
% Qsub:     measurement noise covariance matrix
%
% Output        
% solTx:    transmitter position estimate
% solSx:    scatterer position estimate
% Qlty:     solution quality factor
% QltyTx:   transmitter solution quality factor
% QltySx:   scatterer solution quality factor
%
% The routine can be used for 2D(K=2) or 3D(K=3) localization.
%
% Reference:
% Reference: Y. Zhang and K. C. Ho, "Localization of transmitters and scatterers by single 
% receiver", IEEE Trans. Signal Process., vol. 71, pp. 2267-2282, 2023.
%
% Yang Zhang, K. C. Ho     06-28-2023
%
%       Copyright (C) 2023
%       Computational Intelligence Signal Processing Laboratory
%       University of Missouri
%       Columbia, MO 65211, USA.
%       hod@missouri.edu
%

K=size(TxKn,1);

theta1Tx=angleTx(1);
theta1Sx=angleSx(1);

if K==2
    TriSx=[cos(theta1Sx);sin(theta1Sx)];
    TriTx=[cos(theta1Tx);sin(theta1Tx)];
else
    theta2Tx=angleTx(2);
    theta2Sx=angleSx(2);
    TriSx=[cos(theta2Sx)*cos(theta1Sx);cos(theta2Sx)*sin(theta1Sx);sin(theta2Sx)];
    TriTx=[cos(theta2Tx)*cos(theta1Tx);cos(theta2Tx)*sin(theta1Tx);sin(theta2Tx)];
end

d_sx1=(d(1,1)^2+2*d(1,1)*norm(TxKn))/(2*(d(1,1)+norm(TxKn)-TxKn'*TriSx));
solSx=d_sx1*TriSx;

d_tx2=(2*d(2,1)*d_sx1-d(2,1)^2)/(2*(d(2,1)-d_sx1+solSx'*TriTx));
solTx=d_tx2*TriTx;

rho_d=[zeros(1,K),((solSx-TxKn)/norm(TxKn-solSx)+solSx/norm(solSx))';
    ((solTx-solSx)/norm(solTx-solSx)-solTx/norm(solTx))',((solSx-solTx)/norm(solTx-solSx)+solSx/norm(solSx))'];
rho_angleTx=[[-solTx(2),solTx(1),zeros(1,K-2)]./((norm([solTx(2),solTx(1)]))^2),zeros(1,K)];
rho_angleSx=[zeros(1,K),[-solSx(2),solSx(1),zeros(1,K-2)]./((norm([solSx(2),solSx(1)]))^2)];
if K==3
    rho_angleTx=[rho_angleTx;[-sin(theta2Tx)*cos(theta1Tx),-sin(theta2Tx)*sin(theta1Tx),cos(theta2Tx)]/d_tx2,zeros(1,K)];
    rho_angleSx=[rho_angleSx;zeros(1,K),[-sin(theta2Sx)*cos(theta1Sx),-sin(theta2Sx)*sin(theta1Sx),cos(theta2Sx)]/d_sx1];
end
B_rhoi=[rho_d;rho_angleTx;rho_angleSx];
CovInd=inv(B_rhoi)*Qsub*inv(B_rhoi)';
% FIMInd=B_rhoi'*inv(Qsub)*B_rhoi;

if d_sx1<0
    QltySx=Inf;
else
    QltySx=trace(CovInd(K+1:2*K,K+1:2*K));
end

if d_sx1<0 || d_tx2<0
    QltyTx=Inf;
else
    QltyTx=trace(CovInd(1:K,1:K));
end

if d_sx1<0 || d_tx2<0
    Qlty=Inf;
else
    Qlty=trace(CovInd);
end









